﻿using AutoMapper;
using BizService;
using Microsoft.Extensions.Logging;
using Model.Dto;
using Model.Entities;
using PLCTool.Views.BusinessManageView;
using Prism.Commands;
using Prism.Mvvm;
using Prism.Services.Dialogs;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PLCTool.ViewModels.BusinessManageViewModel
{
    public  class ProjectTestViewModel:BindableBase
    {
        

        private readonly IBasicDeviceService _iBasicDeviceService;
        private readonly IBasicDeviceKindService _iBasicDeviceKindService;
        private readonly IBasicProjectService _iBasicProjectService;
        private readonly IBasicPlcTestSchemeService _basicPlcTestSchemeService;
        private readonly IMapper _mapper;
        private readonly IDialogService _dialog;
        private readonly ILogger _logger;
        
        private ObservableCollection<DeviceTestCardView> allProjectePicList = new ObservableCollection<DeviceTestCardView>();
        public ProjectTestViewModel(IBasicDeviceService iBasicDeviceService, IBasicDeviceKindService iBasicDeviceKindService, IBasicProjectService iBasicProjectService, IBasicPlcTestSchemeService basicPlcTestSchemeService, IMapper mapper, IDialogService dialog, ILogger logger)
        {
            _iBasicDeviceService = iBasicDeviceService;
            _iBasicDeviceKindService = iBasicDeviceKindService;
            _iBasicProjectService = iBasicProjectService;
            _basicPlcTestSchemeService = basicPlcTestSchemeService;
            _mapper = mapper;
            _dialog = dialog;
            _logger = logger;
            OnLoadCommand = new DelegateCommand(OnLoad);
            QueryCommand = new DelegateCommand<object>(Query);
            ResetCommand = new DelegateCommand<object>(Reset);
        }
        #region 私有方法
        private void Reset(object obj)
        {
            ProjectNo = string.Empty;
            ProjectLeader = string.Empty;
            ProjectName = string.Empty;
            StartTime = string.Empty;
            EndTime = string.Empty;
        }

        private void Query(object obj)
        {
            var conditions = (from a in allProjectePicList
                              where (string.IsNullOrEmpty(ProjectNo) ? true : (a.txtProjectNo.Text == ProjectNo))
                              && (string.IsNullOrEmpty(ProjectName) ? true : (a.txtName.Text == ProjectName))
                                && (string.IsNullOrEmpty(ProjectLeader) ? true : (a.txtPProjectLeader.Text == ProjectLeader))
                              select a).ToList();
            ProjectePicList = new ObservableCollection<DeviceTestCardView>(conditions);
        }

        private void OnLoad()
        {
            GetProjectConfig();
        }

        /// <summary>
        /// 获取所有项目
        /// </summary>
        private void GetProjectConfig()
        {
          
            ProjectePicList.Clear();
            allProjectePicList.Clear();
            List<BasDeviceWithSchModel> allDeviceList = new List<BasDeviceWithSchModel>();//所有方案
            var devicelist = _iBasicDeviceService.QueryList();
            var allDeviceKinds = _mapper.Map<List<bas_device>, List<BasDeviceDto>>(devicelist);
            //所有测试方案
            var schlist = _basicPlcTestSchemeService.QueryList();
            var schDtoList = _mapper.Map<List<bas_plc_test_scheme>, List<BasicPlcTestSchemeDto>>(schlist);


            foreach (var item in allDeviceKinds)
            {
                string deviceKind = _iBasicDeviceKindService.Find((int)item.DeviceKindId)?.devicekind_name;
                //在测试方案中查找此设备类型的所有方案
                var schs = schDtoList.FindAll(x => x.DeviceKindName == deviceKind);
                //设备方案
                foreach (var sch in schs)
                {
                    allDeviceList.Add(new BasDeviceWithSchModel()
                    {
                        DeviceId = item.DeviceId,
                        DeviceNo = item.DeviceNo,
                        DeviceName = item.DeviceName,
                        DeviceKindName = _iBasicDeviceKindService.Find((int)item.DeviceKindId)?.devicekind_name,
                        ProjectName = _iBasicProjectService.Find((int)item.ProjectId)?.project_name,
                        SchemeName = sch.SchemeName,
                        SchemeId = sch.SchemeId,
                    });

                }

            }
            //所有新项目
            List<string> projectNames = allDeviceList.Select(x => x.ProjectName).Distinct().ToList();
            foreach (var project in projectNames)
            {
                DeviceTestCardView deviceCard = new DeviceTestCardView();
                deviceCard.txtName.Text = project;
                deviceCard.txtProjectNo.Text = _iBasicProjectService.FindByProjectName(project)?.project_no;
                deviceCard.txtPProjectLeader.Text = _iBasicProjectService.FindByProjectName(project)?.project_leader;
                //赋值
                allProjectePicList.Add(deviceCard);
            }
            foreach (var item in allProjectePicList)
            {
                ProjectePicList.Add(item);
            }
            //计算行数
            if (projectNames.Count % 4 == 0)
            {
                RowsCount = projectNames.Count / 4;
            }
            else
            {
                RowsCount = projectNames.Count / 4 + 1;
            }
            if (RowsCount < 2)
            {
                RowsCount = 2;
            }



        }
        #endregion

        #region 命令绑定
        public DelegateCommand OnLoadCommand { set; get; }
        public DelegateCommand<object> QueryCommand { set; get; }




        public DelegateCommand<object> ResetCommand { set; get; }
       
        #endregion

        #region 数据绑定
        /// <summary>
        /// 设备卡片
        /// </summary>

        private ObservableCollection<DeviceTestCardView> projectPicList=new ObservableCollection<DeviceTestCardView>();
        public ObservableCollection<DeviceTestCardView> ProjectePicList
        {
            get { return projectPicList; }
            set { projectPicList = value; RaisePropertyChanged(); }
        }

        private int rowsCount =2;
        public int RowsCount
        {
            get { return rowsCount; }
            set { rowsCount = value; RaisePropertyChanged(); }
        }

        private string projectNo;
        public string ProjectNo
        {
            get { return projectNo; }
            set { projectNo = value; RaisePropertyChanged(); }
        }

        private string projectName;
        public string ProjectName
        {
            get { return projectName; }
            set { projectName = value; RaisePropertyChanged(); }
        }
        private string projectLeader;
        public string ProjectLeader
        {
            get { return projectLeader; }
            set { projectLeader = value; RaisePropertyChanged(); }
        }

        /// <summary>
        /// 开始时间
        /// </summary>
        private string startTime = DateTime.Now.AddDays(-1).ToString();
        public string StartTime
        {
            get { return startTime; }
            set { startTime = value; RaisePropertyChanged(); }
        }
        private string endTime = DateTime.Now.ToString();
        public string EndTime
        {
            get { return endTime; }
            set { endTime = value; RaisePropertyChanged(); }
        }

        #endregion
    }
}
